﻿Imports Microsoft.ServiceBus
Imports Microsoft.ServiceBus.Description
Imports Microsoft.ServiceBus.Messaging
Imports Microsoft.ServiceBus.Messaging.Filters

'TODO: All of the code for working with the service bus needs to be factored out and into its own 
'      container.
'      
'TODO: Genericize methods for Queue and Topics
'
Partial Friend Class RequestResponse
    Inherits Form

    Public Sub New()
        InitializeComponent()
    End Sub

    Private dsSet As New DataSet()

    'TODO: These belong in config
    Private Const sbNamespace As String = "jofultz"
    Private Const issuerName As String = "owner"
    Private Const issuerKey As String = ""
    Private Const topicName As String = "Inventory Check"
    Private Const subscriptionName As String = "InventoryQuerySubscription"
    Private Const returnQueueName As String = "InventoryResponse"
    Private returnQ As Queue = Nothing
    Private checkQTimer As New Timer()

    Private Inquiries As New List(Of BrokeredMessage)()
    Private InquiryResponses As New InquiryResponseList()


    Private _SbUri As Uri = Nothing
    Private ReadOnly Property ServiceBusUri() As Uri
        Get
            If _SbUri Is Nothing Then
                _SbUri = ServiceBusEnvironment.CreateServiceUri("sb", sbNamespace, String.Empty)
            End If
            Return _SbUri
        End Get

    End Property

    Private _SharedSecretCred As SharedSecretCredential = Nothing
    Private ReadOnly Property ServiceBusCredential() As SharedSecretCredential
        Get
            If _SharedSecretCred Is Nothing Then
                _SharedSecretCred = TransportClientCredentialBase.CreateSharedSecretCredential(issuerName, issuerKey)
            End If
            Return _SharedSecretCred
        End Get
    End Property

    Private _sbnsclient As ServiceBusNamespaceClient = Nothing
    Private ReadOnly Property ServiceBusNSClient() As ServiceBusNamespaceClient
        Get
            If _sbnsclient Is Nothing Then
                _sbnsclient = New ServiceBusNamespaceClient(Me.ServiceBusUri, Me.ServiceBusCredential)
            End If
            Return _sbnsclient
        End Get
    End Property

    Private _msgFactory As MessagingFactory = Nothing
    Private ReadOnly Property MsgFactory() As MessagingFactory
        Get
            If _msgFactory Is Nothing Then
                _msgFactory = MessagingFactory.Create(ServiceBusUri, ServiceBusCredential)
            End If
            Return _msgFactory
        End Get
    End Property

    Private _qClient As QueueClient = Nothing
    Private ReadOnly Property QClient() As QueueClient
        Get
            If _qClient Is Nothing Then
                _qClient = MsgFactory.CreateQueueClient(returnQueueName)
            End If
            Return _qClient
        End Get
    End Property

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs)

        dsSet.ReadXml(Application.StartupPath & "\StoreList.xml")
        cboStore.DataSource = dsSet.Tables("Store")
        cboStore.DisplayMember = "id"
        cboStore.ValueMember = "region"

        checkQTimer.Interval = 3000
        AddHandler checkQTimer.Tick, AddressOf checkQTimer_Tick

    End Sub

    Private Sub checkQTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        Dim receiver As MessageReceiver = QClient.CreateReceiver(ReceiveMode.ReceiveAndDelete)
        Dim NewMessage As BrokeredMessage = Nothing
        Do While receiver.TryReceive(TimeSpan.FromSeconds(1), NewMessage)

            Dim resp As New InquiryResponse()
            resp.CorrelationID = NewMessage.CorrelationId
            resp.Message = NewMessage
            InquiryResponses.Add(resp)
        Loop
    End Sub

    Public Sub CreateTopic(ByVal topicName As String)
        Try
            Dim newTopic As Topic = ServiceBusNSClient.CreateTopic(topicName)
        Catch ex As Exception
            Console.WriteLine(ex.Message)
        End Try
    End Sub

    Public Sub EnumerateTopics()
        Try

            Dim topics As IEnumerable(Of Topic) = ServiceBusNSClient.GetTopics()

            For Each topic As Topic In topics
                Dim path As String = topic.Path
                Console.WriteLine(path)

            Next topic
        Catch ex As Exception
            Console.WriteLine(ex.Message)
        End Try

    End Sub

    Private Sub cmdCheck_Click(ByVal sender As Object, ByVal e As EventArgs)
        cmdCheck.Enabled = False
        lblStatus.Text = "Sending ... "
        'assign data values
        Dim data As New InventoryQueryData()
        data.Sku = txtSKU.Text
        data.RequesterID = cboStore.Text
        data.Region = Me.cboStore.SelectedValue.ToString()

        Dim msgFactory As MessagingFactory = MessagingFactory.Create(Me.ServiceBusUri, Me.ServiceBusCredential)
        Dim topicClient As TopicClient = msgFactory.CreateTopicClient(topicName)
        Dim MsgSender As MessageSender = topicClient.CreateSender()

        Dim msg As BrokeredMessage = BrokeredMessage.CreateMessage(data)
        'add props to message for filtering
        msg.Properties("Region") = data.Region
        msg.Properties("RequesterID") = data.RequesterID

        'set properties for message
        msg.TimeToLive = TimeSpan.FromSeconds(30)
        msg.ReplyTo = returnQueueName
        msg.CorrelationId = Guid.NewGuid().ToString()

        'check if-exists and create response queue
        Try
            returnQ = Me.ServiceBusNSClient.GetQueue(returnQueueName)
        Catch ex As System.Exception
            Debug.WriteLine(ex.Message)
        End Try

        If returnQ Is Nothing Then
            returnQ = Me.ServiceBusNSClient.CreateQueue(returnQueueName)
        End If

        checkQTimer.Enabled = True

        'send message
        MsgSender.Send(msg)

        Inquiries.Add(msg)
        Me.lbRequests.Items.Add(msg.CorrelationId)

        lblStatus.Text = msg.MessageId & " sent!"
        cmdCheck.Enabled = True
    End Sub

    Private Sub lbRequests_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
        Dim correlationId As String = lbRequests.SelectedItem.ToString()

        Dim CorrelatedList As List(Of InquiryResponse) = InquiryResponses.FindAll(Function(resp As InquiryResponse) resp.CorrelationID = correlationId)
        dgvResponses.Rows.Clear()

        For Each resp As InquiryResponse In CorrelatedList
            Dim msg As BrokeredMessage = resp.Message
            Dim Inquiry As CreateInquiry.InventoryQueryData = msg.GetBody(Of InventoryQueryData)()
            dgvResponses.Rows.Add(New Object(4) {msg.MessageId, msg.CorrelationId, Inquiry.ResponderID, Inquiry.Sku, Inquiry.Count})
        Next resp
    End Sub

End Class

Public Class InquiryResponse
    Public CorrelationID As String
    Public Message As BrokeredMessage
End Class

Public Class InquiryResponseList
    Inherits List(Of InquiryResponse)

End Class
