﻿Imports Microsoft.ServiceBus.Messaging
Imports Microsoft.ServiceBus.Messaging.Filters
Imports Microsoft.ServiceBus.Description
Imports Microsoft.ServiceBus

Public Delegate Sub MessageReceivedDelegate(ByVal sender As Object, ByVal e As MessagingEventArgs)

Public Delegate Sub ThreadControlDelegate(ByVal StoreID As String, ByVal ControlCommand As ThreadControlEnum)

Public Enum ThreadControlEnum
    Start = 1
    [Stop] = 0
    Pause = 2
End Enum

Public Class MessagingEventArgs
    Inherits EventArgs

    Public Sub New(ByVal data As Object)
        MessageData = data
    End Sub

    Public MessageData As Object

End Class

'TODO: These belong in config
Public NotInheritable Class Constants
    Public Const sbNamespace As String = "jofultz"
    Public Const issuerName As String = "owner"
    Public Const issuerKey As String = ""
    Public Const topicName As String = "Inventory Check"
    Public Const SubscriptionNameRoot As String = "InventoryQuerySubscription"
End Class

Public NotInheritable Class EventContainer
    Public Shared Event MessageReceivedEvent As MessageReceivedDelegate
    Public Shared Event ThreadControlEvent As ThreadControlDelegate

    Private Sub New()
    End Sub
    Public Shared Sub RaiseThreadControl(ByVal StoreID As String, ByVal ControlCommand As ThreadControlEnum)
        RaiseEvent ThreadControlEvent(StoreID, ControlCommand)
    End Sub

    'TODO:  need to fix to pass the brokered message as the only data
    Public Shared Sub RaiseMessageReceived(ByVal data As Object)
        RaiseEvent MessageReceivedEvent(data, New MessagingEventArgs(data))
    End Sub
End Class

Public Class MonitorThread
    Private credential As SharedSecretCredential = Nothing
    Private sbUri As Uri = Nothing
    Private sbNSClient As ServiceBusNamespaceClient = Nothing
    Private msgFactory As MessagingFactory = Nothing

    Private MonitorTimer As New System.Timers.Timer()
    Private _StoreID As String = Nothing
    Private _MonitorOn As Boolean = False

    Private ReadOnly Property SubscriptionName() As String
        Get
            Return Constants.SubscriptionNameRoot & StoreID
        End Get
    End Property

    Private Property MonitorOn() As Boolean
        Get
            Return _MonitorOn
        End Get
        Set(ByVal value As Boolean)
            _MonitorOn = value
        End Set
    End Property

    Public Property StoreID() As String
        Get
            Return _StoreID
        End Get
        Set(ByVal value As String)
            _StoreID = value
        End Set
    End Property

    Public Region As String = Nothing

    Public Sub New()
        AddHandler EventContainer.ThreadControlEvent, AddressOf EventContainer_ThreadControlEvent

        credential = TransportClientCredentialBase.CreateSharedSecretCredential(Constants.issuerName, Constants.issuerKey)
        sbUri = ServiceBusEnvironment.CreateServiceUri("sb", Constants.sbNamespace, String.Empty)
        sbNSClient = New ServiceBusNamespaceClient(sbUri, credential)

        AddHandler MonitorTimer.Elapsed, AddressOf MonitorTimer_Tick
        MonitorTimer.Interval = 3000

        msgFactory = MessagingFactory.Create(sbUri, credential)

    End Sub

    Private Sub EventContainer_ThreadControlEvent(ByVal StoreID As String, ByVal ControlCommand As ThreadControlEnum)
        Select Case ControlCommand
            Case ThreadControlEnum.Start
                MonitorOn = True
            Case ThreadControlEnum.Stop
                MonitorOn = False
                Me.MonitorTimer.Stop()
                Me.MonitorTimer = Nothing

            Case ThreadControlEnum.Pause
                MonitorOn = False
        End Select
    End Sub

    Public Sub MonitorStart()
        Dim invTopic As Topic = sbNSClient.GetTopic(Constants.topicName)

        Try
            Dim subs As IEnumerable(Of Subscription) = invTopic.GetSubscriptions()

            For Each [sub] As Subscription In subs
                If [sub].Name.Equals(Me.SubscriptionName) Then
                    invTopic.RemoveSubscription([sub].Name)
                    Debug.WriteLine([sub].Name & " - removed")
                End If
            Next [sub]

        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try

        Try

            Dim RegionFilter As New SqlFilterExpression("Region = '" & Me.Region & "' AND RequesterID <> '" & Me.StoreID & "'")
            Dim [sub] As Subscription = invTopic.AddSubscription(SubscriptionName, RegionFilter)

        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try

        MonitorTimer.Start()

    End Sub

    Private Sub MonitorTimer_Tick(ByVal sender As Object, ByVal e As EventArgs)
        MonitorTimer.Stop()

        Dim subClient As SubscriptionClient = msgFactory.CreateSubscriptionClient(Constants.topicName, SubscriptionName)
        Dim msgReceiver As MessageReceiver = subClient.CreateReceiver(ReceiveMode.ReceiveAndDelete)

        msgReceiver.Open()

        Dim recvSuccess As Boolean = False
        Do
            Dim NewMsg As BrokeredMessage
            recvSuccess = msgReceiver.TryReceive(TimeSpan.FromSeconds(3), NewMsg)
            If recvSuccess Then
                Dim qryData As InventoryQueryData = NewMsg.GetBody(Of InventoryQueryData)()

                NewMsg.Properties.Add("ResponderId", Me.StoreID)
                EventContainer.RaiseMessageReceived(NewMsg)

            End If

        Loop While recvSuccess

        If MonitorTimer IsNot Nothing Then
            MonitorTimer.Start()
        End If

    End Sub
End Class