﻿Imports Microsoft.ServiceBus.Messaging
Imports Microsoft.ServiceBus.Messaging.Filters
Imports Microsoft.ServiceBus.Description
Imports Microsoft.ServiceBus

Partial Friend Class TopicMonitorAndResponder
    Inherits Form

    Public Sub New()
        InitializeComponent()
    End Sub

    Private dsSet As New DataSet()
    Private MonitorOn As Boolean = False

    Private MonitorIdTracking As New Dictionary(Of String, Boolean)()
    Private ReceivedMessages As New Dictionary(Of String, BrokeredMessage)()

    Private Sub Form1_Load(ByVal sender As Object, ByVal e As EventArgs)

        dsSet.ReadXml(Application.StartupPath & "\StoreList.xml")
        cboStore.DataSource = dsSet.Tables("Store")
        cboStore.DisplayMember = "id"
        cboStore.ValueMember = "region"

        AddHandler cboStore.SelectedIndexChanged, AddressOf cboStore_SelectedIndexChanged
        AddHandler EventContainer.MessageReceivedEvent, AddressOf EventContainer_MessageReceivedEvent

    End Sub

    Private Sub EventContainer_MessageReceivedEvent(ByVal sender As Object, ByVal e As MessagingEventArgs)
        Dim msg As BrokeredMessage = CType(e.MessageData, BrokeredMessage)
        Me.ReceivedMessages.Add(msg.MessageId, msg)

        UpdateUIElements(msg.MessageId, msg)

    End Sub

    Private Delegate Sub UpdateUIElementsDelegate(ByVal text As String, ByVal data As BrokeredMessage)

    Private Sub UpdateUIElements(ByVal text As String, ByVal data As BrokeredMessage)
        ' InvokeRequired required compares the thread ID of the
        ' calling thread to the thread ID of the creating thread.
        ' If these threads are different, it returns true.
        If Me.dgInquiries.InvokeRequired Then
            Dim callback As New UpdateUIElementsDelegate(AddressOf UpdateUIElements)
            Me.Invoke(callback, New Object() {text, data})
        Else

            Dim RequestObject = data.GetBody(Of InventoryQueryData)()
            RequestObject.ResponderID = data.Properties("ResponderId").ToString()

            Dim values() As Object = {data.MessageId, data.CorrelationId, RequestObject.RequesterID, RequestObject.Sku}

            dgInquiries.Rows.Add(values)
            dgInquiries.Refresh()
        End If
    End Sub

    'TODO:  Stop monitor doesn't work properly; need to debug
    Private Sub cmdStartMonitor_Click(ByVal sender As Object, ByVal e As EventArgs)
        If cmdStartMonitor.Text = "Stop Monitor" Then

            cmdStartMonitor.Text = "Start Monitor"

            EventContainer.RaiseThreadControl(cboStore.Text, ThreadControlEnum.Stop)
            MonitorIdTracking.Remove(cboStore.Text)

            MonitorOn = False

        Else
            'TODO: start thread and connect events
            'TODO: add tracking value to dictionary
            Me.MonitorIdTracking.Add(cboStore.Text, True)

            cmdStartMonitor.Text = "Stop Monitor"

            Dim mt As New MonitorThread()
            mt.StoreID = cboStore.Text
            mt.Region = cboStore.SelectedValue.ToString()
            Dim tstart As New System.Threading.ThreadStart(AddressOf mt.MonitorStart)
            Dim newThread As New System.Threading.Thread(tstart)
            newThread.Start()

            Me.MonitorOn = True

        End If

    End Sub

    Private Sub cboStore_SelectedIndexChanged(ByVal sender As Object, ByVal e As EventArgs)
        Try

            Dim cbo As ComboBox = CType(sender, ComboBox)
            Dim CurrentlyMonitoring As Boolean
            Me.MonitorIdTracking.TryGetValue(cbo.SelectedText, CurrentlyMonitoring)

            If CurrentlyMonitoring Then
                MonitorOn = CurrentlyMonitoring
                Me.cmdStartMonitor.Text = "Stop Monitor"

            Else
                MonitorOn = CurrentlyMonitoring
                Me.cmdStartMonitor.Text = "Start Monitor"
            End If
        Catch ex As Exception
            Debug.WriteLine(ex.Message)
        End Try

    End Sub

    Private Sub cmdSendResponse_Click(ByVal sender As Object, ByVal e As EventArgs)
        Dim SelectedMsgId As String = dgInquiries.SelectedRows(0).Cells(0).Value.ToString()

        'create response message
        Dim msg As BrokeredMessage = ReceivedMessages(SelectedMsgId)
        Dim inquiry As InventoryQueryData = msg.GetBody(Of InventoryQueryData)()

        inquiry.ResponderID = Me.cboStore.Text
        inquiry.Count = Integer.Parse(Me.txtQuantity.Text)

        Dim ResponseMsg As BrokeredMessage = BrokeredMessage.CreateMessage(inquiry)
        ResponseMsg.CorrelationId = msg.CorrelationId

        'send message
        Dim credential As SharedSecretCredential = TransportClientCredentialBase.CreateSharedSecretCredential(Constants.issuerName, Constants.issuerKey)
        Dim sbUri As Uri = ServiceBusEnvironment.CreateServiceUri("sb", Constants.sbNamespace, String.Empty)

        Dim Factory As MessagingFactory = MessagingFactory.Create(sbUri, credential)
        Dim QClient As QueueClient = Factory.CreateQueueClient(msg.ReplyTo)
        Dim Sender_Renamed As MessageSender = QClient.CreateSender()

        Sender_Renamed.Send(ResponseMsg)

    End Sub

    Private Sub dgInquiries_CellContentClick(ByVal sender As Object, ByVal e As DataGridViewCellEventArgs)

    End Sub

End Class
